@@ -16,30 +16,48 @@ format.dots <- function(x,
                         compact = FALSE,
                         show.environments = !compact,
                         show.expressions = !compact,
-                        width = 36,
+                        width = NULL,
                         ...) {
   contents <- mapply(
     x,
     names(x) %||% rep("", length(x)),
-    FUN=function(x, n) {
-      paste0(c(
-        if (is.na(n)) "<NA> = " else if (n == "") "" else c(n, " = "),
-        format.quotation.inner(x,
+    ifelse(seq_along(x) == length(x), "", ", "),
+    SIMPLIFY=FALSE,
+    FUN=function(x, n, sep) {
+      glue(
+        if (is.na(n)) "<NA> = " else if (n == "") "" else paste0(n, " = "),
+        format_quotation_inner(x,
                                compact,
-                               show.environments,
-                               show.expressions,
-                               width)),
-        collapse="")
+                               show.environments=show.environments,
+                               show.expressions=show.expressions,
+                               width=width),
+        sep)
     })
 
-  chars <- paste0("c.dots( ",
-                  paste0(contents, collapse=", "),
-                  " )")
-
-  format.default(chars, ...)
+  chars <- do.call(glue, c("c.dots(", contents, ")"))
+  chars
 }
 
 
+glue <- function(...) {
+  # pastes multiline character vectors "end-to-end".
+  # glue(c("a"), c("b", "c"), c("d", "e"), "f", c("g", "h"))
+  #   -> c("ab", "cd", "efg", "h")
+  args <- list(...)
+  out <- rep("", sum(max(sapply(args, length), 1) - 1) + 1)
+  ptr <- 1
+  for (arg in args) {
+    if(length(arg) > 0) {
+      out[[ptr]] <- paste0(out[[ptr]], arg[[1]])
+    }
+    if (length(arg) > 1) {
+      out[(ptr+1):(ptr+length(arg)-1)] <- arg[-1]
+      ptr <- ptr + length(arg) - 1
+    }
+  }
+  out
+}
+
 #' @rdname format
 #' @description
 #' `format.quotation` constructs a string representation of a
@@ -49,18 +67,20 @@ format.quotation <- function(x,
                              compact = FALSE,
                              show.environments = !compact,
                              show.expressions = !compact,
-                             width = 36,
+                             width = NULL,
                              ...) {
-  chars <- format.quotation.inner(
-    x, compact, show.environments, show.expressions, width = width)
-  format.default(chars, ...)
+  chars <- format_quotation_inner(
+    x, compact = compact,
+    show.environments = show.environments,
+    show.expressions = show.expressions, width = width)
+  chars # format.default(chars, ...)
 }
 
 #' @rdname format
 #' @description
 #' `format.oneline` formats a vector or list so that each item is
 #' displayed on one line. It is similar to [format.AsIs] but tries
-#' harder with language objects. The "oneline" class is used by
+#' harder witlanguage objects. The "oneline" class is used by
 #' [as.data.frame.dots].
 #' @export
 #' @param max.width See [base::format].
@@ -75,14 +95,14 @@ format.oneline <- function(x, max.width=50, width=max.width, ...) {
 #' @export
 #' @rdname format
 print.dots <- function(x, ...) {
-  cat(format(x, ...), "\n")
+  cat(format(x, ...), sep="\n")
   invisible(x)
 }
 
 #' @export
 #' @rdname format
 print.quotation <- function(x, ...) {
-  cat(format(x, ...), "\n")
+  cat(format(x, ...), sep="\n")
   invisible(x)
 }
 
@@ -114,39 +134,50 @@ format.name <- function(x, ...) {
 
 oneline <- function(x) structure(x, class=union("oneline", class(x)))
 
-format.quotation.inner <- function(x,
+format_quotation_inner <- function(x,
                                    compact = FALSE,
                                    show.environments = !compact,
                                    show.expressions = !compact,
-                                   width=36) {
+                                   width=NULL) {
 
   doformat <- function(x) {
-    if (is.language(x)) {
-      c("quote(", dodeparse(x), ")")
+    if (is.language(x) || is.function(x)) {
+      deparse(x,
+              width.cutoff=if (is.null(width)) 60 else width,
+              control=c("keepNA", "useSource", "quoteExpressions"))
     } else {
       one_line(x, format, width=width)
     }
   }
   dodeparse <- function(x) {
     if (is.language(x) || is.character(x) || is.list(x)) {
-      deparse(x, width.cutoff=width, nlines = 1)
+      deparse(x,
+              width.cutoff=if (is.null(width)) 60 else width,
+              control=c("keepNA", "useSource"))
     } else {
       doformat(x)
     }
   }
-  contents <- paste0(c(
+  contents <-
     if(forced(x)) {
-      c(if (is.language(expr(x)) && show.expressions) {
-          c("forced_quo(", dodeparse(expr(x)), ", val=", dodeparse(value(x)))
-        } else {
-          c("forced_quo_(", dodeparse(value(x)))
-        },
-        ")")
+      if (!identical(env(x), emptyenv())
+          #&& show.environments && show.expressions
+          ) {
+        # weird primitive dispatch thing?!
+        glue("weird_quo(", dodeparse(expr(x)),
+             ", env=", dodeparse(env(x)),
+             ", value=", dodeparse(value(x)))
+      } else if (is.language(expr(x)) && show.expressions) {
+        glue("forced_quo(", dodeparse(expr(x)),
+             ", value=", dodeparse(value(x)), ")")
+      } else {
+        glue("forced_quo_(", dodeparse(value(x)), ")")
+      }
     } else {
-      c("quo(",
-        dodeparse(expr(x)),
-        if (show.environments) c(", ", doformat(env(x))),
-        ")")
+      glue("quo(",
+           dodeparse(expr(x)),
+           if (show.environments) paste0(", ", doformat(env(x))),
+           ")")
     }
-  ), collapse="")
+  contents
 }
