@@ -26,6 +26,9 @@ test_that("as.data.frame.dots extracts dots information into a data frame", {
   expect_equal(nrow(as.data.frame(dots())), 0)
   #
   f <- function(...) {
+    g(..., q = x+y)
+  }
+  g <- function(...) {
     as.data.frame(dots(...))
   }
   x <- 2
@@ -251,6 +254,7 @@ test_that("expression mutator", local({
 }))
 
 test_that("dots_envs and mutator", local({
+
   expect_equivalent(envs(dots()), list())
   f1 <- function(...) {
     where <- "e1E"
@@ -283,8 +287,11 @@ test_that("dots_envs and mutator", local({
 #
 test_that("expressions unpacks bytecode", {
   f <- function(x) dots(y=x+1)
+  g <- function(x) dots_exprs(y=x+1)
   f <- compiler::cmpfun(f)
+  g <- compiler::cmpfun(g)
   exprs(f(5)) %is% alist(y=x+1)
+  g(5) %is% alist(y=x+1)
 })
 
 test_that("dots_exprs", {
@@ -307,6 +314,17 @@ test_that("dots_exprs is pointer-stable", {
 
 ## DOTS OBJECT, CALLING AND CURRYING -------------------------------------
 
+test_that("concatenate dots, quotations", {
+    x <- 4
+    y <- 6
+    d <- c(quo(x+y, force=TRUE), quo(x*y))
+    f <- function(...) {
+      set_dots(environment(), d)
+      list(...)
+    }
+    f() %is% list(10, 24)
+})
+
 test_that("do with forced quos -- like do.call(quote=TRUE) without overquoting", {
   x <- 2
   y <- 5
@@ -668,6 +686,11 @@ test_that("function_, make an empty closure", {
   expect_identical(missing_value(), body(r))
   expect_identical(formals(r), NULL)
   expect_equal(formals(r), NULL)
+
+  r <- function_(alist(...=, recursive=FALSE),
+                 quote(c(..., recursive=recursive)))
+  r(list(0, c(1, 2)), list(c(3, 4), 5), recursive=TRUE) %is%
+    0:5
 })
 
 test_that("get_dots returns promise objects", {
